#@version 6/1/2010
'''
# Node class.
# Holds information for each possible pickup zone for children, such as neighbors,
# cost, and if active, number of children at the stop.
#
# @date 5/16/2010
'''
class Node:
    #---Private variables---
    __name = None
    __neighbors = None
    __active = True
    __numChildren = 0
    __x = 0
    __y = 0
    
    #---Constructor---
    def __init__(self, name):
        self.__name = name
        self.__neighbors = {}

    #---Setting methods---
    # addNeighbor function. If neighboring node already exists, it replaces the old cost
    # with the new one, otherwise it inputs the new neighbor with its cost.
    def addNeighbor(self, name, cost):
        if (cost < 0):
            raise Exception('Must have a non-negative value for cost.')
        if name in self.__neighbors:
            del self.__neighbors[name]
        self.__neighbors[name] = cost
    # activate function. Makes this node active (as a possible zone to travel to), and lists
    # the number of children at the stop.
    def activate(self, children):
        self.__active = True
        self.__numChildren = children
    # deactivate function. Sets this node inactive (not a possible pickup zone) and sets the
    # number of children at this node to 0.
    def deactivate(self):
        self.__active = False
        self.__numChildren = 0
    # setXY function. Sets the x y coordinates of this node.
    def setXY(self, x, y):
        self.__x = x
        self.__y = y


    #---Return methods---
    # getName function. Returns the name of this node.
    def getName(self):
        return self.__name
    # isActive function. Returns True if node is a possible pickup zone.
    def isActive(self):
        return self.__active
    # numChildren function. Returns the number of children at this node.
    def numChildren(self):
        return self.__numChildren
    # getNeighbors function. Return a dictionary of neighbors and costs.
    def getNeighbors(self):
        return self.__neighbors
    # getXY function. Returns the x y coordinates of this node.
    def getXY(self):
        return [self.__x, self.__y]


'''
# Node database.
# Holds all information about the nodes in a private list. Can create, delete, and modify nodes
# when prompted.
#
# @date 5/16/2010
'''
class NodeDB:
    #---Private variables---
    __db = None

    #---Constructor---
    def __init__(self):
        self.__db = {}

    #---Setting methods---
    # add function. Constructs a new Node to add to the list with the provided parameters.
    # If a Node with same name exists in the database, an exception is thrown.
    def add(self, name, active = False, children = 0):
        if name in self.__db:
            raise Exception('Node with same name exists in database.')
        node = Node(name)
        if active:
            node.activate(children)
        self.__db[name] = node
    # addNode function. Adds a pre-constructed Node to the database, given that the Node does
    # not already exist.
    def addNode(self, node):
        if node.getName() in self.__db:
            raise Exception('Node with same name exists in database.')
        self.__db[node.getName()] = node
    # delete function. Attempts to delete Node with specified name in database. Throws an
    # exception if Node does not exist.
    def delete(self, name):
        if name in self.__db:
            del self.__db[name]
        else:
            raise Exception('Node does not exist in database.')
    # setNeighbors function. Adds the neighbors in the dictionary to Node of name. Throws
    # an exception of Node name does not exist.
    def setNeighbors(self, name, neighborDict):
        if name not in self.__db:
            raise Exception('Node does not exist in database.')
        else :
            for n in neighborDict:
                self.__db[name].addNeighbor(n, neighborDict[n])
    # addRoute function. Attemps to add a route for nodes name1 and name2 with the given cost.
    def addRoute(self, name1, name2, cost):
        if name1 and name2 not in self.__db:
            raise Exception('Nodes do not exist in database.')
        else:
            self.__db[name1].addNeighbor(name2, cost)
            self.__db[name2].addNeighbor(name1, cost)     
        

    #---Getting methods---
    # get function. Attempts to retrieve node with specified name. Returns a string if node
    # is not found.
    def get(self, name):
        if name in self.__db:
            return self.__db[name]
        else:
            return "Specified node does not exist"
    # getDB function. Returns a list with the names of the nodes within the database.
    def getDB(self):
        return self.__db.keys()


'''
# DBHandler class.
# Uses serialization (pickle) to store and load databases.
#
# @date 5/27/2010
'''
import sys, pickle

class DBHandler():
    __nodeDB = None
    
    # Constructor
    def __init__(self, nodeDB = NodeDB()):
        self.__nodeDB = nodeDB
    # saves the node database into a serializable file.
    def saveDBtoSer(self, nodeFileName = 'nodes.ser'):
        f = open(nodeFileName, 'w')
        pickle.dump(self.__nodeDB, f)
        f.close()
    # saves the node database into a text file.
    def saveDBtoText(self, nodeFileName = 'nodes.txt'):
        f = open(nodeFileName, 'w')
        for node in self.__nodeDB.getDB():
            output = node + '|'
            neighbors = self.__nodeDB.get(node).getNeighbors()
            for neighbor in neighbors.keys():
                output += neighbor+':'+str(neighbors[neighbor]) +','
            output = output[:len(output)-1]
            output += '|'
            output += str(self.__nodeDB.get(node).numChildren()) +'|'
            output += str(self.__nodeDB.get(node).getXY()[0]) +',' +str(self.__nodeDB.get(node).getXY()[1]) +'\n'
            f.write(output)
        f.close()
    # loads the node database from serialized file.
    def loadSerDB(self, nodeFileName):
        f = open(nodeFileName, 'r')
        self.__nodeDB = pickle.load(f)
        f.close()
    # loads the node database from a text file.
    # 'name'|neighbor1:cost,neighbor2:cost|numChildren|X,Y
    def loadTextDB(self, nodeFileName):
        nDB = NodeDB()
        f = open(nodeFileName, 'r')
        temp = f.readline()
        while (temp != ''):
            parts = temp.split('|')
            node = Node(parts[0])
            neighbors = parts[1].split(',')
            nDict = {}
            for n in neighbors:
                twoParts = n.split(':')
                nDict[twoParts[0]] = int(twoParts[1])
            node.activate(int(parts[2]))
            xy = parts[3].split(',')
            node.setXY(int(xy[0]),int(xy[1]))
            nDB.addNode(node)
            nDB.setNeighbors(parts[0], nDict)
            temp = f.readline()
        self.__nodeDB = nDB
        f.close()
    # returns the node database.
    def getDB(self):
        return self.__nodeDB
